@@ -1,11 +1,12 @@
+# Mikel Broström 🔥 Yolo Tracking 🧾 AGPL-3.0 license
+
 from collections import deque
 
 import numpy as np
-import torch
 
 from boxmot.appearance.reid_multibackend import ReIDDetectMultiBackend
 from boxmot.motion.cmc.sof import SparseOptFlow
-from boxmot.motion.kalman_filters.adapters import BotSortKalmanFilterAdapter
+from boxmot.motion.kalman_filters.botsort_kf import KalmanFilter
 from boxmot.trackers.botsort.basetrack import BaseTrack, TrackState
 from boxmot.utils.matching import (embedding_distance, fuse_score,
                                    iou_distance, linear_assignment)
@@ -13,20 +14,20 @@ from boxmot.utils.ops import xywh2xyxy, xyxy2xywh
 
 
 class STrack(BaseTrack):
-    shared_kalman = BotSortKalmanFilterAdapter()
+    shared_kalman = KalmanFilter()
 
-    def __init__(self, tlwh, score, cls, feat=None, feat_history=50):
+    def __init__(self, det, feat=None, feat_history=50):
         # wait activate
-        self._tlwh = np.asarray(tlwh, dtype=np.float32)
+        self.xywh = xyxy2xywh(det[0:4])  # (x1, y1, x2, y2) --> (xc, yc, w, h)
+        self.score = det[4]
+        self.cls = det[5]
+        self.det_ind = det[6]
         self.kalman_filter = None
         self.mean, self.covariance = None, None
         self.is_activated = False
-
-        self.cls = -1
         self.cls_hist = []  # (cls id, freq)
-        self.update_cls(cls, score)
+        self.update_cls(self.cls, self.score)
 
-        self.score = score
         self.tracklet_len = 0
 
         self.smooth_feat = None
@@ -112,11 +113,9 @@ class STrack(BaseTrack):
     def activate(self, kalman_filter, frame_id):
         """Start a new tracklet"""
         self.kalman_filter = kalman_filter
-        self.track_id = self.next_id()
+        self.id = self.next_id()
 
-        self.mean, self.covariance = self.kalman_filter.initiate(
-            self.tlwh_to_xywh(self._tlwh)
-        )
+        self.mean, self.covariance = self.kalman_filter.initiate(self.xywh)
 
         self.tracklet_len = 0
         self.state = TrackState.Tracked
@@ -127,7 +126,7 @@ class STrack(BaseTrack):
 
     def re_activate(self, new_track, frame_id, new_id=False):
         self.mean, self.covariance = self.kalman_filter.update(
-            self.mean, self.covariance, self.tlwh_to_xywh(new_track.tlwh)
+            self.mean, self.covariance, new_track.xywh
         )
         if new_track.curr_feat is not None:
             self.update_features(new_track.curr_feat)
@@ -136,8 +135,10 @@ class STrack(BaseTrack):
         self.is_activated = True
         self.frame_id = frame_id
         if new_id:
-            self.track_id = self.next_id()
+            self.id = self.next_id()
         self.score = new_track.score
+        self.cls = new_track.cls
+        self.det_ind = new_track.det_ind
 
         self.update_cls(new_track.cls, new_track.score)
 
@@ -152,10 +153,8 @@ class STrack(BaseTrack):
         self.frame_id = frame_id
         self.tracklet_len += 1
 
-        new_tlwh = new_track.tlwh
-
         self.mean, self.covariance = self.kalman_filter.update(
-            self.mean, self.covariance, self.tlwh_to_xywh(new_tlwh)
+            self.mean, self.covariance, new_track.xywh
         )
 
         if new_track.curr_feat is not None:
@@ -165,74 +164,22 @@ class STrack(BaseTrack):
         self.is_activated = True
 
         self.score = new_track.score
+        self.cls = new_track.cls
+        self.det_ind = new_track.det_ind
         self.update_cls(new_track.cls, new_track.score)
 
     @property
-    def tlwh(self):
-        """Get current position in bounding box format `(top left x, top left y,
-        width, height)`.
+    def xyxy(self):
+        """Convert bounding box to format `(min x, min y, max x, max y)`, i.e.,
+        `(top left, bottom right)`.
         """
         if self.mean is None:
-            return self._tlwh.copy()
-        ret = self.mean[:4].copy()
-        ret[:2] -= ret[2:] / 2
+            ret = self.xywh.copy()  # (xc, yc, w, h)
+        else:
+            ret = self.mean[:4].copy()  # kf (xc, yc, w, h)
+        ret = xywh2xyxy(ret)
         return ret
 
-    @property
-    def tlbr(self):
-        """Convert bounding box to format `(min x, min y, max x, max y)`, i.e.,
-        `(top left, bottom right)`.
-        """
-        ret = self.tlwh.copy()
-        ret[2:] += ret[:2]
-        return ret
-
-    @property
-    def xywh(self):
-        """Convert bounding box to format `(min x, min y, max x, max y)`, i.e.,
-        `(top left, bottom right)`.
-        """
-        ret = self.tlwh.copy()
-        ret[:2] += ret[2:] / 2.0
-        return ret
-
-    @staticmethod
-    def tlwh_to_xyah(tlwh):
-        """Convert bounding box to format `(center x, center y, aspect ratio,
-        height)`, where the aspect ratio is `width / height`.
-        """
-        ret = np.asarray(tlwh).copy()
-        ret[:2] += ret[2:] / 2
-        ret[2] /= ret[3]
-        return ret
-
-    @staticmethod
-    def tlwh_to_xywh(tlwh):
-        """Convert bounding box to format `(center x, center y, width,
-        height)`.
-        """
-        ret = np.asarray(tlwh).copy()
-        ret[:2] += ret[2:] / 2
-        return ret
-
-    def to_xywh(self):
-        return self.tlwh_to_xywh(self.tlwh)
-
-    @staticmethod
-    def tlbr_to_tlwh(tlbr):
-        ret = np.asarray(tlbr).copy()
-        ret[2:] -= ret[:2]
-        return ret
-
-    @staticmethod
-    def tlwh_to_tlbr(tlwh):
-        ret = np.asarray(tlwh).copy()
-        ret[2:] += ret[:2]
-        return ret
-
-    def __repr__(self):
-        return "OT_{}_({}-{})".format(self.track_id, self.start_frame, self.end_frame)
-
 
 class BoTSORT(object):
     def __init__(
@@ -240,7 +187,8 @@ class BoTSORT(object):
         model_weights,
         device,
         fp16,
-        track_high_thresh: float = 0.45,
+        track_high_thresh: float = 0.5,
+        track_low_thresh: float = 0.1,
         new_track_thresh: float = 0.6,
         track_buffer: int = 30,
         match_thresh: float = 0.8,
@@ -248,7 +196,7 @@ class BoTSORT(object):
         appearance_thresh: float = 0.25,
         cmc_method: str = "sparseOptFlow",
         frame_rate=30,
-        lambda_=0.985,
+        fuse_first_associate: bool = False,
     ):
         self.tracked_stracks = []  # type: list[STrack]
         self.lost_stracks = []  # type: list[STrack]
@@ -257,24 +205,25 @@ class BoTSORT(object):
 
         self.frame_id = 0
 
-        self.lambda_ = lambda_
         self.track_high_thresh = track_high_thresh
+        self.track_low_thresh = track_low_thresh
         self.new_track_thresh = new_track_thresh
+        self.match_thresh = match_thresh
 
         self.buffer_size = int(frame_rate / 30.0 * track_buffer)
         self.max_time_lost = self.buffer_size
-        self.kalman_filter = BotSortKalmanFilterAdapter()
+        self.kalman_filter = KalmanFilter()
 
         # ReID module
         self.proximity_thresh = proximity_thresh
         self.appearance_thresh = appearance_thresh
-        self.match_thresh = match_thresh
 
         self.model = ReIDDetectMultiBackend(
             weights=model_weights, device=device, fp16=fp16
         )
 
         self.cmc = SparseOptFlow()
+        self.fuse_first_associate = fuse_first_associate
 
     def update(self, dets, img):
         assert isinstance(
@@ -296,44 +245,25 @@ class BoTSORT(object):
         lost_stracks = []
         removed_stracks = []
 
-        xyxys = dets[:, 0:4]
-        xywh = xyxy2xywh(xyxys)
+        dets = np.hstack([dets, np.arange(len(dets)).reshape(-1, 1)])
+
+        # Remove bad detections
         confs = dets[:, 4]
-        clss = dets[:, 5]
 
-        classes = clss
-        xyxys = xyxys
-        confs = confs
+        # find second round association detections
+        second_mask = np.logical_and(confs > self.track_low_thresh, confs < self.track_high_thresh)
+        dets_second = dets[second_mask]
 
-        remain_inds = confs > self.track_high_thresh
-        inds_low = confs > 0.1
-        inds_high = confs < self.track_high_thresh
-
-        inds_second = np.logical_and(inds_low, inds_high)
-
-        dets_second = xywh[inds_second]
-        dets = xywh[remain_inds]
-
-        scores_keep = confs[remain_inds]
-        scores_second = confs[inds_second]
-
-        classes_keep = classes[remain_inds]
-        clss_second = classes[inds_second]
-
-        self.height, self.width = img.shape[:2]
+        # find first round association detections
+        first_mask = confs > self.track_high_thresh
+        dets_first = dets[first_mask]
 
         """Extract embeddings """
-        features_keep = self._get_features(dets, img)
+        features_high = self.model.get_features(dets_first[:, 0:4], img)
 
         if len(dets) > 0:
             """Detections"""
-
-            detections = [
-                STrack(xyxy, s, c, f.cpu().numpy())
-                for (xyxy, s, c, f) in zip(
-                    dets, scores_keep, classes_keep, features_keep
-                )
-            ]
+            detections = [STrack(det, f) for (det, f) in zip(dets_first, features_high)]
         else:
             detections = []
 
@@ -353,13 +283,15 @@ class BoTSORT(object):
         STrack.multi_predict(strack_pool)
 
         # Fix camera motion
-        warp = self.cmc.apply(img, dets)
+        warp = self.cmc.apply(img, dets_first)
         STrack.multi_gmc(strack_pool, warp)
         STrack.multi_gmc(unconfirmed, warp)
 
         # Associate with high score detection boxes
         ious_dists = iou_distance(strack_pool, detections)
         ious_dists_mask = ious_dists > self.proximity_thresh
+        if self.fuse_first_associate:
+          ious_dists = fuse_score(ious_dists, detections)
 
         emb_dists = embedding_distance(strack_pool, detections) / 2.0
         emb_dists[emb_dists > self.appearance_thresh] = 1.0
@@ -381,25 +313,9 @@ class BoTSORT(object):
                 refind_stracks.append(track)
 
         """ Step 3: Second association, with low score detection boxes"""
-        # if len(scores):
-        #     inds_high = scores < self.track_high_thresh
-        #     inds_low = scores > self.track_low_thresh
-        #     inds_second = np.logical_and(inds_low, inds_high)
-        #     dets_second = bboxes[inds_second]
-        #     scores_second = scores[inds_second]
-        #     classes_second = classes[inds_second]
-        # else:
-        #     dets_second = []
-        #     scores_second = []
-        #     classes_second = []
-
-        # association the untrack to the low score detections
         if len(dets_second) > 0:
             """Detections"""
-            detections_second = [
-                STrack(STrack.tlbr_to_tlwh(tlbr), s, c)
-                for (tlbr, s, c) in zip(dets_second, scores_second, clss_second)
-            ]
+            detections_second = [STrack(dets_second) for dets_second in dets_second]
         else:
             detections_second = []
 
@@ -476,55 +392,29 @@ class BoTSORT(object):
             self.tracked_stracks, self.lost_stracks
         )
 
-        # output_stracks = [track for track in self.tracked_stracks if track.is_activated]
         output_stracks = [track for track in self.tracked_stracks if track.is_activated]
         outputs = []
         for t in output_stracks:
             output = []
-            tlwh = t.tlwh
-            tid = t.track_id
-            tlwh = np.expand_dims(tlwh, axis=0)
-            xyxy = xywh2xyxy(tlwh)
-            xyxy = np.squeeze(xyxy, axis=0)
-            output.extend(xyxy)
-            output.append(tid)
+            output.extend(t.xyxy)
+            output.append(t.id)
             output.append(t.score)
             output.append(t.cls)
+            output.append(t.det_ind)
             outputs.append(output)
 
         outputs = np.asarray(outputs)
         return outputs
 
-    def _xywh_to_xyxy(self, bbox_xywh):
-        x, y, w, h = bbox_xywh
-        x1 = max(int(x - w / 2), 0)
-        x2 = min(int(x + w / 2), self.width - 1)
-        y1 = max(int(y - h / 2), 0)
-        y2 = min(int(y + h / 2), self.height - 1)
-        return x1, y1, x2, y2
-
-    @torch.no_grad()
-    def _get_features(self, bbox_xywh, ori_img):
-        im_crops = []
-        for box in bbox_xywh:
-            x1, y1, x2, y2 = self._xywh_to_xyxy(box)
-            im = ori_img[y1:y2, x1:x2]
-            im_crops.append(im)
-        if im_crops:
-            features = self.model(im_crops)
-        else:
-            features = np.array([])
-        return features
-
 
 def joint_stracks(tlista, tlistb):
     exists = {}
     res = []
     for t in tlista:
-        exists[t.track_id] = 1
+        exists[t.id] = 1
         res.append(t)
     for t in tlistb:
-        tid = t.track_id
+        tid = t.id
         if not exists.get(tid, 0):
             exists[tid] = 1
             res.append(t)
@@ -534,9 +424,9 @@ def joint_stracks(tlista, tlistb):
 def sub_stracks(tlista, tlistb):
     stracks = {}
     for t in tlista:
-        stracks[t.track_id] = t
+        stracks[t.id] = t
     for t in tlistb:
-        tid = t.track_id
+        tid = t.id
         if stracks.get(tid, 0):
             del stracks[tid]
     return list(stracks.values())
