@@ -1,14 +1,14 @@
+# Mikel Broström 🔥 Yolo Tracking 🧾 AGPL-3.0 license
+
 """
     This script is adopted from the SORT script by Alex Bewley alex@bewley.ai
 """
 
 import numpy as np
-import torch
 
 from boxmot.appearance.reid_multibackend import ReIDDetectMultiBackend
 from boxmot.motion.cmc import get_cmc_method
-from boxmot.motion.kalman_filters.adapters import OCSortKalmanFilterAdapter
-from boxmot.utils import PerClassDecorator
+from boxmot.motion.kalman_filters.deepocsort_kf import KalmanFilter
 from boxmot.utils.association import associate, linear_assignment
 from boxmot.utils.iou import get_asso_func
 
@@ -94,18 +94,21 @@ class KalmanBoxTracker(object):
 
     count = 0
 
-    def __init__(self, bbox, cls, delta_t=3, emb=None, alpha=0, new_kf=False):
+    def __init__(self, det, delta_t=3, emb=None, alpha=0, new_kf=False):
         """
         Initialises a tracker using initial bounding box.
 
         """
         # define constant velocity model
-        self.cls = cls
 
-        self.conf = bbox[-1]
         self.new_kf = new_kf
+        bbox = det[0:5]
+        self.conf = det[4]
+        self.cls = det[5]
+        self.det_ind = det[6]
+
         if new_kf:
-            self.kf = OCSortKalmanFilterAdapter(dim_x=8, dim_z=4)
+            self.kf = KalmanFilter(dim_x=8, dim_z=4)
             self.kf.F = np.array(
                 [
                     # x y w h x' y' w' h'
@@ -192,14 +195,18 @@ class KalmanBoxTracker(object):
 
         self.frozen = False
 
-    def update(self, bbox, cls):
+    def update(self, det):
         """
         Updates the state vector with observed bbox.
         """
-        if bbox is not None:
+
+        if det is not None:
+            bbox = det[0:5]
+            self.conf = det[4]
+            self.cls = det[5]
+            self.det_ind = det[6]
             self.frozen = False
-            self.cls = cls
-            self.conf = bbox[-1]
+
             if self.last_observation.sum() >= 0:  # no previous observation
                 previous_box = None
                 for dt in range(self.delta_t, 0, -1):
@@ -230,7 +237,7 @@ class KalmanBoxTracker(object):
             else:
                 self.kf.update(self.bbox_to_z_func(bbox))
         else:
-            self.kf.update(bbox)
+            self.kf.update(det)
             self.frozen = True
 
     def update_emb(self, emb, alpha=0.9):
@@ -238,7 +245,7 @@ class KalmanBoxTracker(object):
         self.emb /= np.linalg.norm(self.emb)
 
     def get_emb(self):
-        return self.emb.cpu()
+        return self.emb
 
     def apply_affine_correction(self, affine):
         m = affine[:, :2]
@@ -312,7 +319,7 @@ class DeepOCSort(object):
         delta_t=3,
         asso_func="iou",
         inertia=0.2,
-        w_association_emb=0.75,
+        w_association_emb=0.5,
         alpha_fixed_emb=0.95,
         aw_param=0.5,
         embedding_off=False,
@@ -337,9 +344,9 @@ class DeepOCSort(object):
         self.alpha_fixed_emb = alpha_fixed_emb
         self.aw_param = aw_param
         self.per_class = per_class
-        KalmanBoxTracker.count = 0
+        KalmanBoxTracker.count = 1
 
-        self.embedder = ReIDDetectMultiBackend(weights=model_weights, device=device, fp16=fp16)
+        self.model = ReIDDetectMultiBackend(weights=model_weights, device=device, fp16=fp16)
         # "similarity transforms using feature point extraction, optical flow, and RANSAC"
         self.cmc = get_cmc_method('sof')()
         self.embedding_off = embedding_off
@@ -347,7 +354,6 @@ class DeepOCSort(object):
         self.aw_off = aw_off
         self.new_kf_off = new_kf_off
 
-    @PerClassDecorator
     def update(self, dets, img):
         """
         Params:
@@ -357,27 +363,28 @@ class DeepOCSort(object):
         Returns the a similar array, where the last column is the object ID.
         NOTE: The number of objects returned may differ from the number of detections provided.
         """
+        #dets, s, c = dets.data
+        #print(dets, s, c)
         assert isinstance(dets, np.ndarray), f"Unsupported 'dets' input type '{type(dets)}', valid format is np.ndarray"
         assert isinstance(img, np.ndarray), f"Unsupported 'img' input type '{type(img)}', valid format is np.ndarray"
         assert len(dets.shape) == 2, "Unsupported 'dets' dimensions, valid number of dimensions is two"
         assert dets.shape[1] == 6, "Unsupported 'dets' 2nd dimension lenght, valid lenghts is 6"
 
         self.frame_count += 1
-
-        scores = dets[:, 4]
-
-        dets = dets[:, 0:6]
-        remain_inds = scores > self.det_thresh
-        dets = dets[remain_inds]
         self.height, self.width = img.shape[:2]
 
-        # Embedding
+        scores = dets[:, 4]
+        dets = np.hstack([dets, np.arange(len(dets)).reshape(-1, 1)])
+        assert dets.shape[1] == 7
+        remain_inds = scores > self.det_thresh
+        dets = dets[remain_inds]
+
+        # appearance descriptor extraction
         if self.embedding_off or dets.shape[0] == 0:
             dets_embs = np.ones((dets.shape[0], 1))
         else:
             # (Ndets x X) [512, 1024, 2048]
-            # dets_embs = self.embedder.compute_embedding(img, dets[:, :4], tag)
-            dets_embs = self._get_features(dets[:, :4], img)
+            dets_embs = self.model.get_features(dets[:, 0:4], img)
 
         # CMC
         if not self.cmc_off:
@@ -425,7 +432,7 @@ class DeepOCSort(object):
         else:
             stage1_emb_cost = dets_embs @ trk_embs.T
         matched, unmatched_dets, unmatched_trks = associate(
-            dets,
+            dets[:, 0:5],
             trks,
             self.iou_threshold,
             velocities,
@@ -437,7 +444,7 @@ class DeepOCSort(object):
             self.aw_param,
         )
         for m in matched:
-            self.trackers[m[1]].update(dets[m[0], :5], dets[m[0], 5])
+            self.trackers[m[1]].update(dets[m[0], :])
             self.trackers[m[1]].update_emb(dets_embs[m[0]], alpha=dets_alpha[m[0]])
 
         """
@@ -468,7 +475,7 @@ class DeepOCSort(object):
                     det_ind, trk_ind = unmatched_dets[m[0]], unmatched_trks[m[1]]
                     if iou_left[m[0], m[1]] < self.iou_threshold:
                         continue
-                    self.trackers[trk_ind].update(dets[det_ind, :5], dets[det_ind, 5])
+                    self.trackers[trk_ind].update(dets[det_ind, :])
                     self.trackers[trk_ind].update_emb(dets_embs[det_ind], alpha=dets_alpha[det_ind])
                     to_remove_det_indices.append(det_ind)
                     to_remove_trk_indices.append(trk_ind)
@@ -476,13 +483,12 @@ class DeepOCSort(object):
                 unmatched_trks = np.setdiff1d(unmatched_trks, np.array(to_remove_trk_indices))
 
         for m in unmatched_trks:
-            self.trackers[m].update(None, None)
+            self.trackers[m].update(None)
 
         # create and initialise new trackers for unmatched detections
         for i in unmatched_dets:
             trk = KalmanBoxTracker(
-                dets[i, :5],
-                dets[i, 5],
+                dets[i],
                 delta_t=self.delta_t,
                 emb=dets_embs[i],
                 alpha=dets_alpha[i],
@@ -501,33 +507,11 @@ class DeepOCSort(object):
                 d = trk.last_observation[:4]
             if (trk.time_since_update < 1) and (trk.hit_streak >= self.min_hits or self.frame_count <= self.min_hits):
                 # +1 as MOT benchmark requires positive
-                ret.append(np.concatenate((d, [trk.id + 1], [trk.conf], [trk.cls])).reshape(1, -1))
+                ret.append(np.concatenate((d, [trk.id], [trk.conf], [trk.cls], [trk.det_ind])).reshape(1, -1))
             i -= 1
             # remove dead tracklet
             if trk.time_since_update > self.max_age:
                 self.trackers.pop(i)
         if len(ret) > 0:
             return np.concatenate(ret)
-        return np.empty((0, 5))
-
-    def _xywh_to_xyxy(self, bbox_xywh):
-        x, y, w, h = bbox_xywh
-        x1 = max(int(x - w / 2), 0)
-        x2 = min(int(x + w / 2), self.width - 1)
-        y1 = max(int(y - h / 2), 0)
-        y2 = min(int(y + h / 2), self.height - 1)
-        return x1, y1, x2, y2
-
-    @torch.no_grad()
-    def _get_features(self, bbox_xyxy, ori_img):
-        im_crops = []
-        for box in bbox_xyxy:
-            x1, y1, x2, y2 = box.astype(int)
-            im = ori_img[y1:y2, x1:x2]
-            im_crops.append(im)
-        if im_crops:
-            features = self.embedder(im_crops).cpu()
-        else:
-            features = np.array([])
-
-        return features
+        return np.array([])
