@@ -1,6 +1,4 @@
-# vim: expandtab:ts=4:sw=4
-import numpy as np
-
+# Mikel Broström 🔥 Yolo Tracking 🧾 AGPL-3.0 license
 
 class Detection(object):
     """
@@ -26,18 +24,12 @@ class Detection(object):
 
     """
 
-    def __init__(self, tlwh, confidence, feature):
-        self.tlwh = np.asarray(tlwh, dtype=np.float32)
-        self.confidence = float(confidence)
-        self.feature = np.asarray(feature.cpu(), dtype=np.float32)
-
-    def to_tlbr(self):
-        """Convert bounding box to format `(min x, min y, max x, max y)`, i.e.,
-        `(top left, bottom right)`.
-        """
-        ret = self.tlwh.copy()
-        ret[2:] += ret[:2]
-        return ret
+    def __init__(self, tlwh, conf, cls, det_ind, feat):
+        self.tlwh = tlwh
+        self.conf = conf
+        self.cls = cls
+        self.det_ind = det_ind
+        self.feat = feat
 
     def to_xyah(self):
         """Convert bounding box to format `(center x, center y, aspect ratio,
@@ -47,13 +39,3 @@ class Detection(object):
         ret[:2] += ret[2:] / 2
         ret[2] /= ret[3]
         return ret
-
-
-def to_xyah_ext(bbox):
-    """Convert bounding box to format `(center x, center y, aspect ratio,
-    height)`, where the aspect ratio is `width / height`.
-    """
-    ret = bbox.copy()
-    ret[:2] += ret[2:] / 2
-    ret[2] /= ret[3]
-    return ret
